@@ -124,10 +124,6 @@ class InvalidManifestContents(InvalidManifest, ValidationError):
     """Raised when the manifest files are missing or do not match tags, sequences, clusters, extensions etc as required"""
 
 
-class DatasetNameIsNotUnique(InvalidManifest):
-    """Raise when a dataset's name is not unique within its manifest."""
-
-
 # --------------------- Exceptions relating to access of data using the Twine instance ------------------------
 
 # TODO This is related to filtering files from a manifest. Determine whether this belongs here,
@@ -156,6 +152,7 @@ invalid_json_map = {
     "configuration_values": InvalidValuesJson,
     "input_values": InvalidValuesJson,
     "output_values": InvalidValuesJson,
+    "monitor_message": InvalidValuesJson,
     "configuration_manifest": InvalidManifestJson,
     "input_manifest": InvalidManifestJson,
     "output_manifest": InvalidManifestJson,
@@ -168,6 +165,7 @@ invalid_contents_map = {
     "configuration_values": InvalidValuesContents,
     "input_values": InvalidValuesContents,
     "output_values": InvalidValuesContents,
+    "monitor_message": InvalidValuesContents,
     "configuration_manifest": InvalidManifestContents,
     "input_manifest": InvalidManifestContents,
     "output_manifest": InvalidManifestContents,
